==> gnocl-0.5.15.tar.bz2: gnocl-0.5.15/src/plug.c <==
/*
 * $Id: plug.c,v 1.7 2004/02/10 20:57:35 baum Exp $
 *
 * This file implements the plug widget
 *
 * Copyright (c) 2001 - 2004 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
   2003-06: added onDestroy
   2002-02: switched from GnoclWidgetOptions to GnoclOption
   2001-11: Begin of developement
 */

#include "gnocl.h"
#include <gdk/gdkx.h>
#include <string.h>
#include <assert.h>

static GnoclOption plugOptions[] =
{
   { "-child", GNOCL_STRING, NULL },       /* 0 */
   { "-socketID", GNOCL_OBJ, NULL },       /* 1 */
   { "-defaultWidth", GNOCL_INT, "default-width" },     /* these must be */
   { "-defaultHeight", GNOCL_INT, "default-height" },   /* before -visible! */
   { "-visible", GNOCL_BOOL, "visible" },               /* 4 */
   { "-allowShrink", GNOCL_BOOL, "allow-shrink" },
   { "-allowGrow", GNOCL_BOOL, "allow-grow" },
   { "-resizable", GNOCL_BOOL, "resizable" },
   { "-onDestroy", GNOCL_OBJ, "destroy", gnoclOptCommand },  
   { NULL },
};

static const int childIdx    = 0;
static const int socketIDIdx = 1;
static const int visibleIdx  = 4;

static int configure( Tcl_Interp *interp, GtkPlug *plug, 
      GnoclOption options[] )
{
   if( options[childIdx].status == GNOCL_STATUS_CHANGED )
   {
      GtkWidget *childWidget = gnoclChildNotPacked( 
            options[childIdx].val.str, interp );
      if( childWidget == NULL )
         return TCL_ERROR;

      gtk_container_add( GTK_CONTAINER( plug ), childWidget );
   }

   return TCL_OK;
}

static int plugFunc( ClientData data, Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[] )
{

   const char *cmds[] = { "delete", "configure", "getID", NULL };
   enum cmdIdx { DeleteIdx, ConfigureIdx, GetIDIdx };
   int idx;
   GtkPlug *plug = GTK_PLUG( data );

   if( objc < 2 )
   {
      Tcl_WrongNumArgs( interp, 1, objv, "command" );
      return TCL_ERROR;
   }

   if( Tcl_GetIndexFromObj( interp, objv[1], cmds, "command", 
         TCL_EXACT, &idx ) != TCL_OK )
      return TCL_ERROR;

   switch( idx )
   {
      case DeleteIdx:
            return gnoclDelete( interp, GTK_WIDGET( plug ), objc, objv );
      case ConfigureIdx:
            {
               int ret = TCL_ERROR;
               if( plugOptions[socketIDIdx].status == GNOCL_STATUS_CHANGED )
               {
                  Tcl_SetResult( interp, 
                        "Option -socketID cannot be changed after creation",
                        TCL_STATIC );
                  return TCL_ERROR;
               }

               if( gnoclParseAndSetOptions( interp, objc - 1, objv + 1, 
                     plugOptions, G_OBJECT( plug ) ) == TCL_OK )
               {
                  ret = configure( interp, plug, plugOptions );
               }
               gnoclClearOptions( plugOptions );
               return ret;
            }
            break;
      case GetIDIdx:
            {
               long xid;
               Tcl_Obj *val;

               if( objc != 2 )
               {
                  Tcl_WrongNumArgs( interp, 2, objv, NULL );
                  return TCL_ERROR;
               }
               xid = gtk_plug_get_id( plug );
               val = Tcl_NewLongObj( xid );
               Tcl_SetObjResult( interp, val );
            }
            break;
   }

   return TCL_OK;
}

int gnoclPlugCmd( ClientData data, Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[] )
{
   int     ret;
   GtkPlug *plug;
   long    socketId = 0;
   
   if( gnoclParseOptions( interp, objc, objv, plugOptions ) != TCL_OK )
   {
      gnoclClearOptions( plugOptions );
      return TCL_ERROR;
   }

   if( plugOptions[socketIDIdx].status == GNOCL_STATUS_CHANGED )
   {
      if( Tcl_GetLongFromObj( interp, plugOptions[socketIDIdx].val.obj, 
            &socketId ) != TCL_OK )
         return TCL_ERROR;
   }

   plug = GTK_PLUG( gtk_plug_new( socketId ) );

   if( plugOptions[visibleIdx].status == 0 )
      gtk_widget_show( GTK_WIDGET( plug ) );

   ret = gnoclSetOptions( interp, plugOptions, G_OBJECT( plug ), -1 );
   if( ret == TCL_OK )
      ret = configure( interp, plug, plugOptions );
   gnoclClearOptions( plugOptions );

   if( ret != TCL_OK )
   {
      gtk_widget_destroy( GTK_WIDGET( plug ) );
      return TCL_ERROR;
   }

   return gnoclRegisterWidget( interp, GTK_WIDGET( plug ), plugFunc );
}

==> gnocl-0.5.18.tar.bz2: gnocl-0.5.18/src/plug.c <==
/*
 * $Id: plug.c,v 1.8 2004/04/05 18:16:39 baum Exp $
 *
 * This file implements the plug widget
 *
 * Copyright (c) 2001 - 2004 Peter G. Baum  http://www.dr-baum.net
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
   History:
   2003-06: added onDestroy
   2002-02: switched from GnoclWidgetOptions to GnoclOption
   2001-11: Begin of developement
 */

#include "gnocl.h"
#include <gdk/gdkx.h>
#include <string.h>
#include <assert.h>

static GnoclOption plugOptions[] =
{
   { "-socketID", GNOCL_OBJ, NULL },                    /* 0 */
   { "-defaultWidth", GNOCL_INT, "default-width" },     /* these must be */
   { "-defaultHeight", GNOCL_INT, "default-height" },   /* before -visible! */
   { "-visible", GNOCL_BOOL, "visible" },               /* 3 */
   { "-allowShrink", GNOCL_BOOL, "allow-shrink" },
   { "-allowGrow", GNOCL_BOOL, "allow-grow" },
   { "-child", GNOCL_OBJ, "", gnoclOptChild }, 
   { "-resizable", GNOCL_BOOL, "resizable" },
   { "-onDestroy", GNOCL_OBJ, "destroy", gnoclOptCommand },  
   { NULL },
};

static const int socketIDIdx = 0;
static const int visibleIdx  = 3;

static int configure( Tcl_Interp *interp, GtkPlug *plug, 
      GnoclOption options[] )
{

   return TCL_OK;
}

static int plugFunc( ClientData data, Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[] )
{

   const char *cmds[] = { "delete", "configure", "getID", NULL };
   enum cmdIdx { DeleteIdx, ConfigureIdx, GetIDIdx };
   int idx;
   GtkPlug *plug = GTK_PLUG( data );

   if( objc < 2 )
   {
      Tcl_WrongNumArgs( interp, 1, objv, "command" );
      return TCL_ERROR;
   }

   if( Tcl_GetIndexFromObj( interp, objv[1], cmds, "command", 
         TCL_EXACT, &idx ) != TCL_OK )
      return TCL_ERROR;

   switch( idx )
   {
      case DeleteIdx:
            return gnoclDelete( interp, GTK_WIDGET( plug ), objc, objv );
      case ConfigureIdx:
            {
               int ret = TCL_ERROR;
               if( plugOptions[socketIDIdx].status == GNOCL_STATUS_CHANGED )
               {
                  Tcl_SetResult( interp, 
                        "Option -socketID cannot be changed after creation",
                        TCL_STATIC );
                  return TCL_ERROR;
               }

               if( gnoclParseAndSetOptions( interp, objc - 1, objv + 1, 
                     plugOptions, G_OBJECT( plug ) ) == TCL_OK )
               {
                  ret = configure( interp, plug, plugOptions );
               }
               gnoclClearOptions( plugOptions );
               return ret;
            }
            break;
      case GetIDIdx:
            {
               long xid;
               Tcl_Obj *val;

               if( objc != 2 )
               {
                  Tcl_WrongNumArgs( interp, 2, objv, NULL );
                  return TCL_ERROR;
               }
               xid = gtk_plug_get_id( plug );
               val = Tcl_NewLongObj( xid );
               Tcl_SetObjResult( interp, val );
            }
            break;
   }

   return TCL_OK;
}

int gnoclPlugCmd( ClientData data, Tcl_Interp *interp,
      int objc, Tcl_Obj * const objv[] )
{
   int     ret;
   GtkPlug *plug;
   long    socketId = 0;
   
   if( gnoclParseOptions( interp, objc, objv, plugOptions ) != TCL_OK )
   {
      gnoclClearOptions( plugOptions );
      return TCL_ERROR;
   }

   if( plugOptions[socketIDIdx].status == GNOCL_STATUS_CHANGED )
   {
      if( Tcl_GetLongFromObj( interp, plugOptions[socketIDIdx].val.obj, 
            &socketId ) != TCL_OK )
         return TCL_ERROR;
   }

   plug = GTK_PLUG( gtk_plug_new( socketId ) );

   if( plugOptions[visibleIdx].status == 0 )
      gtk_widget_show( GTK_WIDGET( plug ) );

   ret = gnoclSetOptions( interp, plugOptions, G_OBJECT( plug ), -1 );
   if( ret == TCL_OK )
      ret = configure( interp, plug, plugOptions );
   gnoclClearOptions( plugOptions );

   if( ret != TCL_OK )
   {
      gtk_widget_destroy( GTK_WIDGET( plug ) );
      return TCL_ERROR;
   }

   return gnoclRegisterWidget( interp, GTK_WIDGET( plug ), plugFunc );
}
